﻿using System;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.ModelConfiguration;
using System.Data.Entity.ModelConfiguration.Conventions;
using System.Diagnostics;

using GamesStore.Entities;

using Microsoft.AspNet.Identity.EntityFramework;

namespace GamesStore.DataProviders
{
    public class GamesDbContext : IdentityDbContext<User>
    {
        public GamesDbContext()
            : base("name=DefaultConnection")
        {
            Database.Log = log => Debug.WriteLine(log, "GamesDbContext");
            Database.CommandTimeout = 240;
            Database.Connection.StateChange += Connection_StateChange;

            Configuration.AutoDetectChangesEnabled = true;
            Configuration.LazyLoadingEnabled = true;
            Configuration.ProxyCreationEnabled = true;
        }

        private void Connection_StateChange(object sender, System.Data.StateChangeEventArgs e)
        {
            if (e.OriginalState == e.CurrentState)
                return;
            if(e.CurrentState == ConnectionState.Closed || e.CurrentState == ConnectionState.Broken)
            {
                Debug.WriteLine($"{DateTime.Now.ToString("yyyy-MM-dd_HH:mm:ss")}: Connection was {e.OriginalState}, but now it is {e.CurrentState}!");
            }
        }

        public virtual DbSet<Game> Games { get; set; }

        public virtual DbSet<Studio> Studios { get; set; }

        public virtual DbSet<Developer> Developers { get; set; }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            modelBuilder.Configurations.Add(new GameConfiguration());
            modelBuilder.Configurations.Add(new StudioConfiguration());
            modelBuilder.Conventions.Add(new IdConvention());

            base.OnModelCreating(modelBuilder);
        }

        private class GameConfiguration : EntityTypeConfiguration<Game>
        {
            public GameConfiguration()
            {
                Property(game => game.Name).HasMaxLength(150);
            }
        }

        private class StudioConfiguration : EntityTypeConfiguration<Studio>
        {
            public StudioConfiguration()
            {
                Property(studio => studio.Name).HasMaxLength(250);
                ToTable("CustomStudios");
            }
        }


        private class IdConvention : Convention
        {
            public IdConvention()
            {
                Properties<Guid>()
                .Where(property => property.Name == "Id")
                .Configure(configuration => configuration
                    .IsKey()
                    .HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity));
            }
        }
    }
}