namespace GamesStore.DataProviders.GamesDbMigrations
{
    using System;
    using System.Data.Entity.Migrations;
    using Entities;
    using Microsoft.AspNet.Identity.EntityFramework;
    using Microsoft.AspNet.Identity;
    using System.Collections.Generic;

    internal sealed class GamesDbConfiguration : DbMigrationsConfiguration<GamesDbContext>
    {
        public GamesDbConfiguration()
        {
            AutomaticMigrationsEnabled = false;
            AutomaticMigrationDataLossAllowed = true;
            MigrationsDirectory = @"GamesDbMigrations\Migrations";
        }

        protected override void Seed(GamesDbContext context)
        {
            var user = new User
            {
                UserName = "Slavo",                
            };

            var userManager = new ApplicationUserManager(new UserStore<User>(context));
            userManager.Create(user, "test123");

            var games = SeedGames(context);

            var developers = SeedDevelopers(context);

            SeedStudios(context, games, developers);
        }

        private IReadOnlyList<Developer> SeedDevelopers(GamesDbContext context)
        {
            var developers = new[] {
                new Developer()
                {
                    FirstName = "John",
                    LastName = "Romero"
                },
                new Developer()
                {
                    FirstName = "Sandy",
                    LastName = "Petersen"
                },
                new Developer()
                {
                    FirstName = "Shawn",
                    LastName = "Green"
                },
                new Developer()
                {
                    FirstName = "Tom",
                    LastName = "Hall"
                }
            };

            context.Developers.AddOrUpdate(x => x.LastName, developers);
            context.SaveChanges();

            return developers;
        }

        private static void SeedStudios(GamesDbContext context, IReadOnlyList<Game> games, IReadOnlyList<Developer> developers)
        {
            context.Studios.AddOrUpdate(
                x => x.Name,
                new Studio()
                {
                    Name = "Nintendo Forever",
                    FoundationYear = 1970,
                    Games =
                    {
                        games[0],
                        games[1]
                    },
                    Developers =
                    {
                        developers[3],
                        developers[0],
                        developers[1]
                    }
                },
                new Studio()
                {
                    Name = "Kill 'em all",
                    FoundationYear = 2000,
                    Games =
                    {
                        games[2],
                        games[3],
                        games[4]
                    },
                    Developers =
                    {
                        developers[2]
                    }
                });
        }

        private static IReadOnlyList<Game> SeedGames(GamesDbContext context)
        {
            var games = new[]
            {
                new Game()
                {
                    Name = "Super Mario Bros.",
                    Genre = GameGenreEnum.Action,
                    PublishedOn = new DateTime(1985, 9, 13)
                },
                new Game() {
                    Name = "The Secret of Monkey Island",
                    Genre = GameGenreEnum.Adventure,
                    PublishedOn = new DateTime(1990, 10, 1)
                },
                new Game()
                {
                    Name = "Plants vs. Zombies",
                    Genre = GameGenreEnum.Arcade,
                    PublishedOn = new DateTime(2009, 5, 5),
                },
                new Game()
                {
                    Name = "Wolfenstein 3D",
                    Genre = GameGenreEnum.FirstPersonShooter,
                    PublishedOn = new DateTime(1992, 5, 5),
                },
                new Game()
                {
                    Name = "Wolfenstein: The Old Blood",
                    Genre = GameGenreEnum.FirstPersonShooter,
                }
            };

            context.Games.AddOrUpdate(x => x.Name, games);
            context.SaveChanges();

            return games;
        }
    }
}
