﻿using System;
using System.Collections.Generic;
using System.Linq;

using NUnit.Framework;

using GamesStore.Entities;

namespace GamesStore.Repository.Tests
{
    class GameQueryableExtensionsTests
    {
        [TestFixture]
        public class FilterByGenreTests
        {
            [Test]
            public void FilterByGenre_Null_ReturnsNull()
            {
                IQueryable<Game> games = null;

                Assert.That(() => games.FilterByGenre("Action"), Throws
                    .InstanceOf<ArgumentNullException>()
                    .With.Property("ParamName").EqualTo("games"), "Should have returned null.");
            }

            [Test]
            public void FilterByGenre_Filter_GamesAreFiltered()
            {
                var games = new List<Game>
                {
                    new Game { Genre = GameGenreEnum.Action },
                    new Game { Genre = GameGenreEnum.Adventure },
                    new Game { Genre = GameGenreEnum.Arcade }
                }.AsQueryable();

                var actual = games.FilterByGenre("Adventure");

                Assert.AreEqual(1, actual.Count(), "Should have returned only one matching game.");
                Assert.AreEqual(GameGenreEnum.Adventure, actual.First().Genre, "Filtered games should be only adventure.");
            }

            [Test]
            public void FilterByGenres_NotMatchingGames_FilteredOut()
            {
                var games = new List<Game>
                {
                    new Game { Genre = GameGenreEnum.Action },
                    new Game { Genre = GameGenreEnum.Arcade}
                }.AsQueryable();

                var actual = games.FilterByGenre("Strategy;FirstPersonShooter");

                CollectionAssert.IsEmpty(actual, "All games should have been filtered out.");
            }
        }
    }
}
