﻿using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

using Moq;
using NUnit.Framework;

using GamesStore.Entities;
using GamesStore.DataProviders;
using GamesStore.Repository.Tests.Infrastructure;

namespace GamesStore.Repository.Tests
{
    class GamesServiceTests
    {
        [TestFixture]
        public class GetGamesByGenreAsyncTests
        {
            [Test]
            public async void GetGamesByGenreAsync_Games_AreFiltered()
            {
                var data = new List<Game>
                {
                    new Game { Name = "A", Genre = GameGenreEnum.Action },
                    new Game { Name = "B", Genre = GameGenreEnum.Adventure },
                    new Game { Name = "C", Genre = GameGenreEnum.Arcade }
                }.AsQueryable();

                var mockSet = new Mock<DbSet<Game>>();
                mockSet.As<IQueryable<Game>>().Setup(m => m.Provider)
                    //.Returns(data.Provider);
                    .Returns(new TestDbAsyncQueryProvider<Game>(data.Provider));
                mockSet.As<IQueryable<Game>>().Setup(m => m.Expression).Returns(data.Expression);
                mockSet.As<IQueryable<Game>>().Setup(m => m.ElementType).Returns(data.ElementType);
                mockSet.As<IQueryable<Game>>().Setup(m => m.GetEnumerator()).Returns(data.GetEnumerator());

                var mockContext = new Mock<GamesDbContext>()
                {
                    // Allow calling the real underlying object on not mocked members.
                    // CallBase = true
                };
                mockContext.Setup(m => m.Games).Returns(mockSet.Object);

                var service = new GamesService(mockContext.Object);

                var actual = await service.GetGamesByGenreAsync("Adventure");

                CollectionAssert.IsSubsetOf(actual, data, "Should have selected some games from the original collection");
                Assert.That(actual, Is.All.Property("Genre").EqualTo(GameGenreEnum.Adventure), "All selected games should have genre specified in the filter.");

            }
        }

    }
}
